import re
import unicodedata
import nltk
from nltk.corpus import stopwords
import pandas as pd
from sklearn.utils import shuffle

nltk.download('stopwords')


def __unicode_to_ascii(s):
    """

    :param s:
    :return:
    """
    return ''.join(c for c in unicodedata.normalize('NFD', s)
                   if unicodedata.category(c) != 'Mn')


def __clean_stopwords_shortwords(w):
    """

    :param w:
    :return:
    """
    stopwords_list = stopwords.words('english')
    words = w.split()
    clean_words = [
        word for word in words
        if (word not in stopwords_list) and len(word) > 2
    ]
    return " ".join(clean_words)


def __preprocess_sentence(w):
    """

    :param w:
    :return:
    """
    w = __unicode_to_ascii(w.lower().strip())
    w = re.sub(r'https*\S+', ' ', w)
    w = re.sub(r"([?.!,¿])", r" ", w)
    w = re.sub(r'[" "]+', " ", w)
    w = re.sub(r"[^a-zA-Z?.!,¿]+", " ", w)
    w = __clean_stopwords_shortwords(w)
    w = re.sub(r'@\w+', '', w)
    return w


def clean_dataframe(df):
    """

    :param df:
    :return:
    """
    print('File has {} rows and {} columns'.format(df.shape[0], df.shape[1]))
    df = df.rename(columns={0: "label", 1: "text"})
    # Drop NaN valuues, if any
    df = df.dropna()
    # Reset index after dropping the columns/rows with NaN values
    df = df.reset_index(drop=True)
    # Shuffle the dataset
    df = shuffle(df)
    # Print all the unique labels in the dataset
    print('Available labels: ', df.label.unique())
    # Clean the text column using preprocess_sentence function defined above
    df['text'] = df['text'].map(__preprocess_sentence)
    return df



import tensorflow as tf
import pandas as pd
import numpy as np
import tensorflow.keras
from sklearn.model_selection import train_test_split
from transformers import *
from transformers import BertTokenizer, TFBertModel, BertConfig


class ModelTrainer:

    def __init__(self, model_name, save_path, data):
        """

        :param model_name:
        :param save_path:
        :param data:
        """
        self.callbacks = []
        self.input_ids = []
        self.attention_masks = []
        self.labels = []
        self.train_inp = None
        self.val_inp = None
        self.train_label = None
        self.val_label = None
        self.train_mask = None
        self.val_mask = None
        self.history = None
        self.save_path = save_path
        self.tokenizer = BertTokenizer.from_pretrained(model_name)
        self.data = data
        num_classes = len(data.label.unique())
        print("There are " + str(num_classes) + " unique Classes in the Dataset")
        self.model = TFBertForSequenceClassification.from_pretrained(model_name, num_labels=num_classes)

    def auto_train(self):
        """

        :return:
        """
        self.tokenize_data()
        self.train_test_split()
        self.define_model()
        self.train_model()

    def tokenize_data(self, max_length=128):
        """

        :param max_length:
        :return:
        """
        print("Tokenize Data \n")
        self.data['gt'] = self.data['label'].map({
            'Chemistry': 0,
            'Computer Sience': 1,
            'Medicine': 2
        })
        self.data.head()

        sentences = self.data['text']
        self.labels = self.data['gt']

        for sent in sentences:
            tokenizer = self.tokenizer
            bert_inp = tokenizer.encode_plus(sent,add_special_tokens=True,max_length=max_length,return_attention_mask=True)
            self.input_ids.append(bert_inp['input_ids'])
            self.attention_masks.append(bert_inp['attention_mask'])

        self.input_ids = np.asarray(self.input_ids, dtype="object")
        self.attention_masks = np.array(self.attention_masks, dtype="object")
        self.labels = np.array(self.labels)

        self.input_ids = pd.DataFrame(self.input_ids.tolist())
        self.attention_masks = pd.DataFrame(self.attention_masks.tolist())
        self.labels = pd.DataFrame(self.labels.tolist())
        self.input_ids = self.input_ids.fillna(0).astype('int32')
        self.attention_masks = self.attention_masks.fillna(0).astype('int32')
        print('Input shape {} Attention mask shape {} Input label shape {}'.format(self.input_ids.shape,
                                                                                   self.attention_masks.shape,
                                                                                   self.labels.shape))

    def train_test_split(self):
        """

        :return:
        """
        print("Train Test Split \n")
        self.train_inp, self.val_inp, self.train_label, self.val_label, self.train_mask, self.val_mask = train_test_split(
            self.input_ids, self.labels, self.attention_masks, test_size=0.2)

    def define_model(self):
        """

        :return:
        """
        print("Define Model \n")
        log_dir = 'logs/'
        model_save_path = 'logs/bert_model.h5'
        """ self.callbacks = [
            tensorflow.keras.callbacks.ModelCheckpoint(filepath=model_save_path,
                                                       save_weights_only=True,
                                                       monitor='val_loss',
                                                       mode='min',
                                                       save_best_only=True),
            tensorflow.keras.callbacks.TensorBoard(log_dir=log_dir)
        ] """
        print('\nBert Model', self.model.summary())

        loss = tensorflow.keras.losses.SparseCategoricalCrossentropy(from_logits=True)
        metric = tensorflow.keras.metrics.SparseCategoricalAccuracy('accuracy')
        optimizer = tensorflow.keras.optimizers.Adam(learning_rate=2e-5, epsilon=1e-08)

        self.model.compile(loss=loss, optimizer=optimizer, metrics=[metric])

    def train_model(self):
        """

        :return:
        """
        print("Train Model \n")
        self.history = self.model.fit([self.train_inp, self.train_mask],
                                      self.train_label,
                                      verbose=1,
                                      batch_size=32,
                                      epochs=4,
                                      callbacks=self.callbacks,
                                      validation_data=([self.val_inp, self.val_mask], self.val_label))



import io
import os

# import pdfplumber
import fitz  # PyMuPDF
import nltk
import pandas as pd
import pyocr.builders
from PIL import Image
from nltk.corpus import stopwords
from pdf2image import convert_from_path
from pdfminer import high_level as pdf_extractor
import pickle
import pkg_resources

nltk.download('stopwords')
stop_words = stopwords.words("english")
lookup = {0: "Chemistry", 1: "Computer Sience", 2: "Medicine"}
__name__ = "hammer_sh"

def load_saved_text():
    """

    :return:
    """
    stream = pkg_resources.resource_filename(__name__, 'sample_data/articles.pickle')
    with open(stream, 'rb') as file:
        df = pickle.load(file)
    return df


def extract_images(rootdir):
    """

    :param rootdir:
    :return:
    """
    data = []
    files = __get_files(rootdir);

    for idx, category in enumerate(files):
        for file in category:
            data.append([lookup[idx], __extract_images(file)])
    df = pd.DataFrame(data)
    return df


def __extract_images(path):
    """

    :param path:
    :return:
    """
    file = fitz.open(path)
    images = []
    for page_index in range(len(file)):
        # get the page itself
        page = file[page_index]
        image_list = page.getImageList()
        # printing number of images found in this page
        if image_list:
            print(f"[+] Found a total of {len(image_list)} images in page {page_index + 1}")
        else:
            print("[!] No images found on page", page_index + 1)
        for image_index, img in enumerate(page.getImageList(), start=1):
            # get the XREF of the image
            xref = img[0]
            # extract the image bytes
            base_image = file.extractImage(xref)
            image_bytes = base_image["image"]
            # get the image extension
            image_ext = base_image["ext"]
            # load it to PIL
            image = Image.open(io.BytesIO(image_bytes))
            images.append(image)
            # save it to local disk
            image.save(open(f"image{page_index + 1}_{image_index}.{image_ext}", "wb"))
    return images


def __get_files(rootdir):
    """

    :param rootdir:
    :return:
    """
    subfolders = [f.path for f in os.scandir(rootdir) if f.is_dir()]
    files = []
    for subfolder in subfolders:
        files.append(__list_files(subfolder))
    return files


def extract_text(rootdir, ocr=False):
    """

    :param rootdir:
    :param ocr:
    :return:
    """
    files = __get_files(rootdir)
    data = []
    if ocr is True:
        for idx, category in enumerate(files):
            for file in category:
                data.append([lookup[idx], __extract_text_ocr(file)])
    else:
        for idx, category in enumerate(files):
            for file in category:
                data.append([lookup[idx], __extract_text(file)])
    df = pd.DataFrame(data)
    return df


def __extract_text(path):
    """

    :param path:
    :return:
    """
    text = pdf_extractor.extract_text(path)
    return text


def __extract_text_ocr(path):
    """

    :param path:
    :return:
    """
    pages = convert_from_path(path, 500)
    image_counter = 1
    for page in pages:
        filename = "page_" + str(image_counter) + ".jpg"
        # Save the image of the page in system
        page.save("temp/" + filename, 'JPEG')
        # Increment the counter to update filename
        image_counter = image_counter + 1

    # Variable to get count of total number of pages
    filelimit = image_counter - 1

    # Iterate from 1 to total number of pages
    complete_text = ""
    for i in range(1, filelimit + 1):
        filename = "page_" + str(i) + ".jpg"

        # Recognize the text as string in image using pytesserct
        tools = pyocr.get_available_tools()[0]
        text = tools.image_to_string(Image.open(filename), builder=pyocr.tesseract.builders.TextBuilder())
        text = text.replace('-\n', '')
        complete_text = complete_text + text

    return complete_text


def __list_files(directory):
    """

    :param directory:
    :return:
    """
    r = []
    for root, dirs, files in os.walk(directory):
        for name in files:
            r.append(os.path.join(root, name))
    return r



from sklearn.metrics import confusion_matrix, f1_score, classification_report


""" def show_tensorboard():
    %load_ext tensorboard
    %tensorboard --logdir {log_dir} """


class Evaluator:
    def __init__(self, model_trainer):
        """

        :param model_trainer:
        """
        self.model = model_trainer.model
        self.pred_labels =[]
        self.val_inp = model_trainer.val_inp
        self.val_label = model_trainer.val_label
        self.val_mask = model_trainer.val_mask
        self.tokenizer = model_trainer.tokenizer

    def evaluate_model(self):
        """

        :return:
        """
        preds = self.model.predict([self.val_inp, self.val_mask], batch_size=32)
        self.pred_labels = preds.logits.argmax(axis=1)

        f1 = f1_score(self.val_label, self.pred_labels, average='micro')
        print('F1 score', f1)
        print('Classification Report')
        target_names = ['Computer Sience', 'Chemistry', 'Medicine']
        print(classification_report(self.val_label, self.pred_labels, target_names=target_names))

    def show_comparison(self):
        """

        :return:
        """
        lookup = {'0': "Chemistry", '1': "Computer Sience", '2': "Medicine"}
        val_label_list = self.val_label.values.tolist()
        val_inp_list = self.val_inp.values.tolist()
        for idx, label in enumerate(self.pred_labels):

            if label == val_label_list[idx][0]:
                icon = "✔️"
            else:
                icon = "❌"

            print(icon, "Predicted: ", lookup[str(label)], "\t=> Actual: ", lookup[str(val_label_list[idx][0])])
            abstract = self.tokenizer.convert_ids_to_tokens(ids=val_inp_list[idx], skip_special_tokens='False')
            print("Abstract: ", ' '.join(abstract))
            print("\n")


import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="hammer_sh", # Replace with your own username
    version="0.0.15",
    author="Sebastian Hammer",
    author_email="hammerse65450@th-nuernberg.de",
    description="A package containing useful methods for my masterthesis",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/hammerse65450/hammer_sh",
    packages=setuptools.find_packages(),
    include_package_data=True,
    package_data={'': ['sample_data/*.pickle']},
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        'pandas',
        'nltk',
        'numpy',
        'matplotlib',
        'transformers',
        'tensorflow',
        'sklearn',
        'requests',
        'bs4',
        'PyMuPDF',
        'pyocr',
        'pdf2image',
        'pdfminer',
        'tqdm',
    ],
    python_requires='>=3.6',
)



import requests
from bs4 import BeautifulSoup
import pandas as pd
from tqdm.notebook import trange, tqdm
import pickle
import pkg_resources
__name__ = "hammer_sh"
base_url = "https://link.springer.com"


def load_saved_abstracts():
    """

    :return:
    """
    stream = pkg_resources.resource_filename(__name__, 'sample_data/abstracts.pickle')
    with open(stream, 'rb') as file:
        df = pickle.load(file)
    return df


def extract_abstracts():
    """

    :return:
    """
    urls_to_search = {
        "Computer Sience": 'https://link.springer.com/search?facet-discipline=%22Computer+Science%22&facet-content'
                           '-type=%22Article%22&facet-language=%22En%22',
        "Chemistry": 'https://link.springer.com/search?facet-discipline=%22Chemistry%22&facet-content-type=%22Article'
                     '%22&facet-language=%22En%22',
        'Medicine': 'https://link.springer.com/search?facet-discipline=%22Medicine+%26+Public+Health%22&facet-content'
                    '-type=%22Article%22&facet-language=%22En%22 '
    }

    data = {}
    for key, url_to_search in urls_to_search.items():
        data[key] = __scrape_over_pages(url_to_search, 50)

    df = []
    for category, abstracts in data.items():
        for abstract in abstracts:
            df.append([category, abstract])

    df = pd.DataFrame(df)
    df.head()
    return df


def __get_abstracts(article_links):
    """

    :param article_links:
    :return:
    """
    abstracts = []
    with tqdm(total=len(article_links)) as progress_bar:
        for link in article_links:
            url = base_url + link['href']
            response = requests.get(url)
            soup = BeautifulSoup(response.text, "html.parser")
            abstract = soup.find("div", {"id": "Abs1-content"})
            if abstract is not None:
                abstracts.append(abstract.text)
            progress_bar.update(1)
    return abstracts


def __analyze_page(url):
    """

    :param url:
    :return:
    """
    response = requests.get(url)
    soup = BeautifulSoup(response.text, "html.parser")
    article_links = soup.findAll('a', class_="title", href=True)
    next_page_object = soup.findAll('a', class_="next", href=True)
    next_page_link = base_url + next_page_object[0]['href']
    return article_links, next_page_link


def __scrape_over_pages(start_url, number_of_results):
    """

    :param start_url:
    :param number_of_results:
    :return:
    """
    results = []
    url = start_url
    while len(results) < number_of_results:
        article_links, url = __analyze_page(url)
        abstracts = __get_abstracts(article_links)
        results = results + abstracts
    return results
